#!/usr/bin/env python3
# This file is placed in the Public Domain.


"stub"


__author__ = "Bart Thate <skullbonesandnumber@gmail.com>"


import setuptools


if __name__ == "__main__":
    setuptools.setup()
